BUFF_FLASH_TIME = 0.75;
BUFF_FLASH_STATE = 0;
BUFF_ALPHA_VALUE = 1;
BUFF_MAX_ALPHA = 1.2;
BUFF_MIN_ALPHA = 0.3;
BUFFS_COUNT = 0;
DEBUFFS_COUNT = 0;

adBuffBar = {}; -- the addon's namespace
adBuffBarDB = { -- the addon's datatable
	["BuffLength"] = {}, -- list of all buffs maximum length captured
	["DebuffLength"] = {}, -- list of all debuffs maximum length captured
};

function adBuffBar:TimeDisplay(t)
	m = math.floor(t/60);
	s = math.ceil(t-m*60);
	if m >= 60 then
		return os.date("%H:%M", t);
	elseif m >= 1 then
		return os.date("%M:%S", t);
	elseif m < 1 then
		return string.format("%.1f",t).."s";
	end
end

function adBuffBar:UpdateBar(this)
	local name, texture, count;
	if ( this.debuff ) then
		name, texture, count = UnitDebuff("player", this:GetID());
	else
		name, texture, count = UnitBuff("player", this:GetID());
	end
	if ( not name ) then
		this:Hide();
	else
		if ( count < 1 ) then
			count = "";
		end
		getglobal(this:GetName().."Icon"):SetTexture(texture);
		getglobal(this:GetName().."Name"):SetText(name);
		getglobal(this:GetName().."Count"):SetText(count);
		this:SetAlpha(1.0);
		this:Show();
		if (adBuffBarConfig["SNAP"] == "TOP") then
			adBuffBarDebuff1:ClearAllAnchors();
			if (DEBUFFS_COUNT+12) > 0 then
				adBuffBarBuff1:ClearAllAnchors();
				adBuffBarBuff1:SetAnchor("BOTTOM","TOP","adBuffBarDebuff"..(DEBUFFS_COUNT+14),0,0);
			else
				adBuffBarBuff1:ClearAllAnchors();
				adBuffBarBuff1:SetAnchor("TOP","TOP","adBuffBarDebuff1",0,0);
			end
		elseif (adBuffBarConfig["SNAP"] == "BOTTOM") then
			adBuffBarBuff1:ClearAllAnchors();
			if (BUFFS_COUNT+24) > 0 then
				adBuffBarDebuff1:ClearAllAnchors();
				adBuffBarDebuff1:SetAnchor("BOTTOM","TOP","adBuffBarBuff"..(BUFFS_COUNT+26),0,0);
			else
				adBuffBarDebuff1:ClearAllAnchors();
				adBuffBarDebuff1:SetAnchor("TOP","TOP","adBuffBarBuff1",0,0);
			end
		end
	end
	
	if ( GameTooltip:IsOwned(this) ) then
		if ( this.debuff ) then
			GameTooltip:SetUnitDebuff("player", this:GetID());
		else
			GameTooltip:SetUnitBuff("player", this:GetID());
		end
	end
	
	adBuffBar:OnBarUpdate(this);
end

function adBuffBar:OnBarLoad(this)
	this:RegisterForClicks("RightButton");
	this:RegisterEvent("UNIT_BUFF_CHANGED");
	this:RegisterEvent("VARIABLES_LOADED");
	adBuffBar:UpdateBar(this);
end

function adBuffBar:OnBarEvent(this, event)
	if ( (not this.debuff and this:GetID() <= adBuffBarConfig["BUFFS_AMOUNT"]) or (this.debuff and this:GetID() <= adBuffBarConfig["DEBUFFS_AMOUNT"]) ) then
		if ( event == "UNIT_BUFF_CHANGED" or event == "VARIABLES_LOADED" ) then
			if ( arg1 == "player" ) then
				adBuffBar:UpdateDBInfo(this:GetID(), this.debuff);
				adBuffBar:UpdateBar(this);
			end
		end
	else
		this:Hide();
	end
end

function adBuffBar:OnBarUpdate(this, elapsedTime)
	local name, leftTime, maxTime;
	local spark = getglobal(this:GetName().."Spark");
	local sparkPosition;
	
	if ( this.debuff ) then
		name = UnitDebuff("player", this:GetID());
		leftTime = UnitDebuffLeftTime("player", this:GetID());
		if ( adBuffBarConfig["COLORS"] ) then
			this:SetBarColor(adBuffBarConfig["COLORS"]["DEBUFFS"].r, adBuffBarConfig["COLORS"]["DEBUFFS"].g, adBuffBarConfig["COLORS"]["DEBUFFS"].b);
		end
		if ( adBuffBarDB["DebuffLength"][name] ) then
			maxTime = adBuffBarDB["DebuffLength"][name];
		else
			maxTime = 1;
		end
	else
		name = UnitBuff("player", this:GetID());
		leftTime = UnitBuffLeftTime("player", this:GetID());
		if ( leftTime and adBuffBarConfig["WARNING_TIME"] ) then
			if ( leftTime and leftTime < adBuffBarConfig["WARNING_TIME"] and adBuffBarConfig["FLASH"] and not adBuffBarConfig["FLASH_TO_RED"] ) then
				this:SetAlpha(BUFF_ALPHA_VALUE);
			elseif ( leftTime and leftTime < adBuffBarConfig["WARNING_TIME"] and adBuffBarConfig["FLASH"] and adBuffBarConfig["FLASH_TO_RED"] ) then
				this:SetBarColor(BUFF_ALPHA_VALUE, 0.5, 0.5);
			else
				this:SetAlpha(1.0);
				if ( adBuffBarConfig["COLORS"] ) then
					this:SetBarColor(adBuffBarConfig["COLORS"]["BUFFS"].r, adBuffBarConfig["COLORS"]["BUFFS"].g, adBuffBarConfig["COLORS"]["BUFFS"].b);
				end
			end
			
			if ( adBuffBarDB["BuffLength"][name] ) then
				maxTime = adBuffBarDB["BuffLength"][name];
			else
				maxTime = 1;
			end
		end
	end
	
	if ( leftTime ) then
		if ( adBuffBarConfig["FILLS"] ) then
			local barWidth;
			if ( adBuffBarConfig["SHRINK"] ) then
				barWidth = 30;
			else
				barWidth = adBuffBarConfig["BARS_WIDTH"];
			end
			if ( adBuffBarConfig["INVERT_FILLING"] ) then
				this:SetValue(100-leftTime/maxTime*100);
				sparkPosition = (1-leftTime/maxTime) * barWidth;
			else
				this:SetValue(leftTime/maxTime*100);
				sparkPosition = leftTime/maxTime * barWidth;
			end
		else
			sparkPosition = 0;
			this:SetValue(100);
		end
		
		if ( sparkPosition < 0 ) then
			sparkPosition = 0;
		end
		
		if ( adBuffBarConfig["FILLS"] ) then
			spark:Show();
		else
			spark:Hide();
		end
		spark:ClearAllAnchors();
		spark:SetAnchor("CENTER", "LEFT", getglobal(this:GetName()), sparkPosition, 0);
	else
		if ( adBuffBarConfig["COLORS"] ) then
			this:SetBarColor(adBuffBarConfig["COLORS"]["BUFFS"].r, adBuffBarConfig["COLORS"]["BUFFS"].g, adBuffBarConfig["COLORS"]["BUFFS"].b);
		end
		spark:Hide();
	end
	
	adBuffBar:UpdateBarDuration(this, leftTime);
end

function adBuffBar:UpdateBarDuration(buffButton, leftTime)
	local duration = getglobal(buffButton:GetName().."Duration");
	if ( leftTime and adBuffBarConfig["WARNING_TIME"] ) then
		if ( adBuffBarConfig["SHORTTIME"] ) then
			duration:SetText(SecondsToTimeAbbrev(leftTime));
		else
			duration:SetText(adBuffBar:TimeDisplay(leftTime));
		end
		if ( leftTime < adBuffBarConfig["WARNING_TIME"] ) then
			duration:SetColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		else
			duration:SetColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		end
		duration:Show();
	else
		duration:Hide();
	end
end

function adBuffBar:OnUpdateFrame(this, elapsedTime)
	if ( BUFF_FLASH_STATE ) then
		BUFF_ALPHA_VALUE = BUFF_ALPHA_VALUE - elapsedTime * BUFF_FLASH_TIME;
		if ( BUFF_ALPHA_VALUE < BUFF_MIN_ALPHA ) then
			BUFF_ALPHA_VALUE = BUFF_MIN_ALPHA;
			BUFF_FLASH_STATE = nil;
		end
	else
		BUFF_ALPHA_VALUE = BUFF_ALPHA_VALUE + elapsedTime * BUFF_FLASH_TIME;
		if ( BUFF_ALPHA_VALUE > BUFF_MAX_ALPHA ) then
			BUFF_ALPHA_VALUE = BUFF_MAX_ALPHA;
			BUFF_FLASH_STATE = 1;
		end
	end
end

function adBuffBar:UpdateDBInfo(id, isDebuff)
	local name, leftTime, maxTime;
	if ( isDebuff ) then
		name = UnitDebuff("player", id);
		leftTime = UnitDebuffLeftTime("player", id);
	else
		leftTime = UnitBuffLeftTime("player", id);
		name = UnitBuff("player", id);
	end
	if ( name and leftTime ) then
		if ( isDebuff ) then
			if ( adBuffBarDB["DebuffLength"][name] ) then
				maxTime = adBuffBarDB["DebuffLength"][name];
			else
				maxTime = 0;
			end
			if ( leftTime > maxTime ) then
				adBuffBarDB["DebuffLength"][name] = leftTime;
			end
		else
			if ( adBuffBarDB["BuffLength"][name] ) then
				maxTime = adBuffBarDB["BuffLength"][name];
			else
				maxTime = 0;
			end
			if ( leftTime > maxTime ) then
				adBuffBarDB["BuffLength"][name] = leftTime;
			end
		end
	end
	SaveVariables("adBuffBarDB");
end

function adBuffBar:SetSkin(skin)
	for i=1,24 do
		local bar = getglobal("adBuffBarBuff"..i.."BarTexture");
		bar:SetTexture(skin);
	end
	for i=1,12 do
		local bar = getglobal("adBuffBarDebuff"..i.."BarTexture");
		bar:SetTexture(skin);
	end
	
	adBuffBarConfig["SKIN"] = skin;
end
